# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import OrderedDict
from ddt import ddt
import json
import mock
import tempfile

from pylxd.deprecated import connection

from pylxd.deprecated.tests import annotated_data
from pylxd.deprecated.tests import fake_api
from pylxd.deprecated.tests import LXDAPITestBase


@ddt
@mock.patch.object(connection.LXDConnection, 'get_object',
                   return_value=('200', fake_api.fake_operation()))
class LXDAPIContainerTestObject(LXDAPITestBase):

    def test_list_containers(self, ms):
        ms.return_value = ('200', fake_api.fake_container_list())
        self.assertEqual(
            ['trusty-1'],
            self.lxd.container_list())
        ms.assert_called_once_with('GET',
                                   '/1.0/containers')

    @annotated_data(
        ('STOPPED', False),
        ('STOPPING', False),
        ('ABORTING', False),
        ('RUNNING', True),
        ('STARTING', True),
        ('FREEZING', True),
        ('FROZEN', True),
        ('THAWED', True),
    )
    def test_container_running(self, status, running, ms):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_container_state(status))
            self.assertEqual(running, self.lxd.container_running('trusty-1'))
            ms.assert_called_once_with('GET',
                                       '/1.0/containers/trusty-1/state')

    def test_container_init(self, ms):
        self.assertEqual(ms.return_value, self.lxd.container_init('fake'))
        ms.assert_called_once_with('POST',
                                   '/1.0/containers',
                                   '"fake"')

    def test_container_update(self, ms):
        self.assertEqual(ms.return_value,
                         self.lxd.container_update('trusty-1',
                                                   'fake'))
        ms.assert_called_once_with('PUT',
                                   '/1.0/containers/trusty-1',
                                   '"fake"')

    def test_container_state(self, ms):
        ms.return_value = ('200', fake_api.fake_container_state('RUNNING'))
        self.assertEqual(ms.return_value, self.lxd.container_state('trusty-1'))
        ms.assert_called_with('GET',
                              '/1.0/containers/trusty-1/state')

    @annotated_data(
        ('start', 'start'),
        ('stop', 'stop'),
        ('suspend', 'freeze'),
        ('resume', 'unfreeze'),
        ('reboot', 'restart'),
    )
    def test_container_actions(self, method, action, ms):
        self.assertEqual(
            ms.return_value,
            getattr(self.lxd, 'container_' + method)('trusty-1', 30))
        ms.assert_called_once_with('PUT',
                                   '/1.0/containers/trusty-1/state',
                                   json.dumps({'action': action,
                                               'force': True,
                                               'timeout': 30, }))

    def test_container_destroy(self, ms):
        self.assertEqual(
            ms.return_value, self.lxd.container_destroy('trusty-1'))
        ms.assert_called_once_with('DELETE',
                                   '/1.0/containers/trusty-1')

    def test_container_log(self, ms):
        ms.return_value = ('200', fake_api.fake_container_log())
        self.assertEqual(
            'fake log', self.lxd.get_container_log('trusty-1'))
        ms.assert_called_once_with('GET',
                                   '/1.0/containers/trusty-1?log=true')

    def test_container_config(self, ms):
        ms.return_value = ('200', fake_api.fake_container_state('fake'))
        self.assertEqual(
            {'status': 'fake'}, self.lxd.get_container_config('trusty-1'))
        ms.assert_called_once_with('GET',
                                   '/1.0/containers/trusty-1?log=false')

    def test_container_info(self, ms):
        ms.return_value = ('200', fake_api.fake_container_state('fake'))
        self.assertEqual(
            {'status': 'fake'}, self.lxd.container_info('trusty-1'))
        ms.assert_called_once_with('GET',
                                   '/1.0/containers/trusty-1/state')

    def test_container_migrate(self, ms):
        ms.return_value = ('200', fake_api.fake_container_migrate())
        self.assertEqual(
            ('200', {'type': 'sync', 'status': 'Success',
                     'metadata': {'criu': 'fake_criu', 'fs':
                                  'fake_fs', 'control':
                                  'fake_control'}, 'operation':
                     '/1.0/operations/1234', 'status_code': 200}),
            self.lxd.container_migrate('trusty-1'))
        ms.assert_called_once_with('POST',
                                   '/1.0/containers/trusty-1',
                                   '{"migration": true}')

    def test_container_publish(self, ms):
        ms.return_value = ('200', fake_api.fake_operation())
        self.assertEqual(
            ms.return_value, self.lxd.container_publish('trusty-1'))
        ms.assert_called_once_with('POST',
                                   '/1.0/images',
                                   '"trusty-1"')

    def test_container_put_file(self, ms):
        temp_file = tempfile.NamedTemporaryFile()
        ms.return_value = ('200', fake_api.fake_standard_return())
        self.assertEqual(
            ms.return_value, self.lxd.put_container_file('trusty-1',
                                                         temp_file.name,
                                                         'dst_file'))
        ms.assert_called_once_with(
            'POST',
            '/1.0/containers/trusty-1/files?path=dst_file',
            body=b'',
            headers={'X-LXD-gid': 0, 'X-LXD-mode': 0o644, 'X-LXD-uid': 0})

    def test_list_snapshots(self, ms):
        ms.return_value = ('200', fake_api.fake_snapshots_list())
        self.assertEqual(
            ['/1.0/containers/trusty-1/snapshots/first'],
            self.lxd.container_snapshot_list('trusty-1'))
        ms.assert_called_once_with('GET',
                                   '/1.0/containers/trusty-1/snapshots')

    @annotated_data(
        ('create', 'POST', '', ('fake config',), ('"fake config"',)),
        ('info', 'GET', '/first', ('first',), ()),
        ('rename', 'POST', '/first',
         ('first', 'fake config'), ('"fake config"',)),
        ('delete', 'DELETE', '/first', ('first',), ()),
    )
    def test_snapshot_operations(self, method, http, path,
                                 args, call_args, ms):
        self.assertEqual(
            ms.return_value,
            getattr(self.lxd,
                    'container_snapshot_' + method)('trusty-1', *args))
        ms.assert_called_once_with(http,
                                   '/1.0/containers/trusty-1/snapshots' +
                                   path,
                                   *call_args)

    def test_container_run_command(self, ms):
        data = OrderedDict((
            ('command', ['/fake/command']),
            ('interactive', False),
            ('wait-for-websocket', False),
            ('environment', {'FAKE_ENV': 'fake'})
        ))

        self.assertEqual(
            ms.return_value,
            self.lxd.container_run_command('trusty-1', *data.values()))
        self.assertEqual(1, ms.call_count)
        self.assertEqual(
            ms.call_args[0][:2],
            ('POST', '/1.0/containers/trusty-1/exec'))
        self.assertEqual(
            json.loads(ms.call_args[0][2]),
            dict(data)
        )


@ddt
@mock.patch.object(connection.LXDConnection, 'get_object',
                   return_value=('200', fake_api.fake_container_list()))
class LXDAPIContainerTestStatus(LXDAPITestBase):

    def test_container_defined(self, ms):
        self.assertTrue(self.lxd.container_defined('trusty-1'))
        ms.assert_called_once_with('GET', '/1.0/containers')


@ddt
@mock.patch.object(connection.LXDConnection, 'get_raw',
                   return_value='fake contents')
class LXDAPIContainerTestRaw(LXDAPITestBase):

    def test_container_file(self, ms):
        self.assertEqual(
            'fake contents', self.lxd.get_container_file('trusty-1',
                                                         '/file/name'))
        ms.assert_called_once_with(
            'GET', '/1.0/containers/trusty-1/files?path=/file/name')



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
from ddt import ddt
import mock
from six.moves import builtins
from six.moves import cStringIO
import unittest

from pylxd.deprecated import connection
from pylxd.deprecated import exceptions
from pylxd.deprecated import image

from pylxd.deprecated.tests import annotated_data
from pylxd.deprecated.tests import fake_api
from pylxd.deprecated.tests import LXDAPITestBase


@ddt
@mock.patch.object(connection.LXDConnection, 'get_object',
                   return_value=('200', fake_api.fake_image_info()))
class LXDAPIImageTestObject(LXDAPITestBase):

    list_data = (
        ('list', (), ()),
        ('search', ({'foo': 'bar'},), ('foo=bar',)),
    )

    @annotated_data(*list_data)
    def test_list_images(self, method, args, call_args, ms):
        ms.return_value = ('200', fake_api.fake_image_list())
        self.assertEqual(
            ['trusty'], getattr(self.lxd, 'image_' + method)(*args))
        ms.assert_called_once_with('GET', '/1.0/images', *call_args)

    @annotated_data(*list_data)
    def test_list_images_fail(self, method, args, call_args, ms):
        ms.side_effect = exceptions.PyLXDException
        self.assertRaises(exceptions.PyLXDException,
                          getattr(self.lxd, 'image_' + method),
                          *args)
        ms.assert_called_once_with('GET', '/1.0/images', *call_args)

    @annotated_data(
        (True, (('200', fake_api.fake_image_info()),)),
        (False, exceptions.APIError("404", 404)),
    )
    def test_image_defined(self, expected, side_effect, ms):
        ms.side_effect = side_effect
        self.assertEqual(expected, self.lxd.image_defined('test-image'))
        ms.assert_called_once_with('GET', '/1.0/images/test-image')

    @annotated_data(
        ('APIError', exceptions.APIError("500", 500), exceptions.APIError),
        ('PyLXDException', exceptions.PyLXDException,
         exceptions.PyLXDException)
    )
    def test_image_defined_fail(self, tag, side_effect, expected, ms):
        ms.side_effect = side_effect
        self.assertRaises(expected,
                          self.lxd.image_defined, ('test-image',))
        ms.assert_called_once_with('GET', '/1.0/images/test-image')

    def test_image_info(self, ms):
        self.assertEqual({
            'image_upload_date': (datetime.datetime
                                  .fromtimestamp(1435669853)
                                  .strftime('%Y-%m-%d %H:%M:%S')),
            'image_created_date': 'Unknown',
            'image_expires_date': 'Unknown',
            'image_public': False,
            'image_size': '63MB',
            'image_fingerprint': '04aac4257341478b49c25d22cea8a6ce'
                                 '0489dc6c42d835367945e7596368a37f',
            'image_architecture': 'x86_64',
        }, self.lxd.image_info('test-image'))
        ms.assert_called_once_with('GET', '/1.0/images/test-image')

    def test_image_info_fail(self, ms):
        ms.side_effect = exceptions.PyLXDException
        self.assertRaises(exceptions.PyLXDException,
                          self.lxd.image_info, ('test-image',))
        ms.assert_called_once_with('GET', '/1.0/images/test-image')

    dates_data = (
        ('upload', (datetime.datetime.fromtimestamp(1435669853)
                    .strftime('%Y-%m-%d %H:%M:%S'))),
        ('create', 'Unknown'),
        ('expire', 'Unknown'),
    )

    @annotated_data(*dates_data)
    def test_image_date(self, method, expected, ms):
        self.assertEqual(expected, getattr(
            self.lxd, 'image_{}_date'.format(method))('test-image',
                                                      data=None))
        ms.assert_called_once_with('GET', '/1.0/images/test-image')

    @annotated_data(*dates_data)
    def test_image_date_fail(self, method, expected, ms):
        ms.side_effect = exceptions.PyLXDException
        self.assertRaises(exceptions.PyLXDException, getattr(
            self.lxd,
            'image_{}_date'.format(method)),
            'test-image',
            data=None)
        ms.assert_called_once_with('GET', '/1.0/images/test-image')


@ddt
@mock.patch.object(connection.LXDConnection, 'get_status', return_value=True)
class LXDAPIImageTestStatus(LXDAPITestBase):
    operations_data = (
        ('delete', 'DELETE', '/test-image', ('test-image',), ()),
        ('update', 'PUT', '/test-image', ('test-image', 'fake',), ('"fake"',)),
        ('rename', 'POST', '/test-image',
         ('test-image', 'fake',), ('"fake"',)),
    )

    @annotated_data(*operations_data)
    def test_image_operations(self, method, http, path, args, call_args, ms):
        self.assertTrue(
            getattr(self.lxd, 'image_' + method)(*args))
        ms.assert_called_once_with(
            http,
            '/1.0/images' + path,
            *call_args
        )

    @annotated_data(*operations_data)
    def test_image_operations_fail(self, method, http, path,
                                   args, call_args, ms):
        ms.side_effect = exceptions.PyLXDException
        self.assertRaises(exceptions.PyLXDException,
                          getattr(self.lxd, 'image_' + method),
                          *args)
        ms.assert_called_once_with(
            http,
            '/1.0/images' + path,
            *call_args
        )


@mock.patch.object(connection.LXDConnection, 'get_object',
                   return_value=('200', fake_api.fake_image_info()))
class LXDAPAPIImageTestUpload(LXDAPITestBase):
    @mock.patch.object(builtins, 'open', return_value=cStringIO('fake'))
    def test_image_upload_file(self, mo, ms):
        self.assertTrue(self.lxd.image_upload(path='/fake/path'))
        mo.assert_called_once_with('/fake/path', 'rb')
        ms.assert_called_once_with('POST', '/1.0/images', 'fake', {})


@mock.patch.object(connection.LXDConnection, 'get_raw')
class LXDAPIImageTestRaw(LXDAPITestBase):

    def test_image_export(self, ms):
        ms.return_value = 'fake contents'
        self.assertEqual('fake contents', self.lxd.image_export('fake'))
        ms.assert_called_once_with('GET', '/1.0/images/fake/export')

    def test_image_export_fail(self, ms):
        ms.side_effect = exceptions.PyLXDException
        self.assertRaises(exceptions.PyLXDException,
                          self.lxd.image_export, 'fake')
        ms.assert_called_once_with('GET', '/1.0/images/fake/export')


@ddt
@mock.patch.object(connection.LXDConnection, 'get_object',
                   return_value=(200, fake_api.fake_image_info()))
class LXDAPIImageInfoTest(unittest.TestCase):

    def setUp(self):
        super(LXDAPIImageInfoTest, self).setUp()
        self.image = image.LXDImage()

    info_list = (
        ('permission', False),
        ('size', 63),
        ('fingerprint', '04aac4257341478b49c25d22cea8a6ce'
                        '0489dc6c42d835367945e7596368a37f'),
        ('architecture', 'x86_64'),
    )

    @annotated_data(*info_list)
    def test_info_no_data(self, method, expected, mc):
        self.assertEqual(expected,
                         (getattr(self.image, 'get_image_' + method)
                          ('test-image', data=None)))
        mc.assert_called_once_with('GET', '/1.0/images/test-image')

    @annotated_data(*info_list)
    def test_info_no_data_fail(self, method, expected, mc):
        mc.side_effect = exceptions.PyLXDException
        self.assertRaises(exceptions.PyLXDException,
                          getattr(self.image, 'get_image_' + method),
                          'test-image',
                          data=None)

    @annotated_data(
        ('permission_true', 'permission', {'public': 0}, False),
        ('permission_false', 'permission', {'public': 1}, True),
        ('size', 'size', {'size': 52428800}, 50),
        ('fingerprint', 'fingerprint', {'fingerprint': 'AAAA'}, 'AAAA'),
        *[('architecture_' + v, 'architecture', {'architecture': k}, v)
          for k, v in image.image_architecture.items()]
    )
    def test_info_data(self, tag, method, metadata, expected, mc):
        self.assertEqual(
            expected, getattr(self.image, 'get_image_' + method)
            ('test-image', data=metadata))
        self.assertFalse(mc.called)

    @annotated_data(
        ('permission', 'permission', {}, KeyError),
        ('size', 'size', {'size': 0}, exceptions.ImageInvalidSize),
        ('size', 'size', {'size': -1}, exceptions.ImageInvalidSize),
        ('fingerprint', 'fingerprint', {}, KeyError),
        ('architecture', 'architecture', {}, KeyError),
        ('architecture_invalid', 'architecture',
         {'architecture': -1}, KeyError)
    )
    def test_info_data_fail(self, tag, method, metadata, expected, mc):
        self.assertRaises(expected,
                          getattr(self.image, 'get_image_' + method),
                          'test-image',
                          data=metadata)
        self.assertFalse(mc.called)



import functools
import importlib
import inspect


class BaseManager(object):
    """A BaseManager class for handling collection operations."""

    @property
    def manager_for(self):  # pragma: no cover
        raise AttributeError(
            "Manager class requires 'manager_for' attribute")

    def __init__(self, *args, **kwargs):
        manager_for = self.manager_for
        module = '.'.join(manager_for.split('.')[0:-1])
        obj = manager_for.split('.')[-1]
        target_module = importlib.import_module(module)
        target = getattr(target_module, obj)

        methods = inspect.getmembers(target, predicate=inspect.ismethod)
        for name, method in methods:
            func = functools.partial(method, *args, **kwargs)
            setattr(self, name, func)
        return super(BaseManager, self).__init__()


class CertificateManager(BaseManager):
    manager_for = 'pylxd.models.Certificate'


class ContainerManager(BaseManager):
    manager_for = 'pylxd.models.Container'


class ImageManager(BaseManager):
    manager_for = 'pylxd.models.Image'


class NetworkManager(BaseManager):
    manager_for = 'pylxd.models.Network'


class OperationManager(BaseManager):
    manager_for = 'pylxd.models.Operation'


class ProfileManager(BaseManager):
    manager_for = 'pylxd.models.Profile'


class SnapshotManager(BaseManager):
    manager_for = 'pylxd.models.Snapshot'



# Copyright (c) 2016 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from pylxd import exceptions

from integration.testing import IntegrationTestCase


class TestContainers(IntegrationTestCase):
    """Tests for `Client.containers`."""

    def test_get(self):
        """A container is fetched by name."""
        name = self.create_container()
        self.addCleanup(self.delete_container, name)

        container = self.client.containers.get(name)

        self.assertEqual(name, container.name)

    def test_all(self):
        """A list of all containers is returned."""
        name = self.create_container()
        self.addCleanup(self.delete_container, name)

        containers = self.client.containers.all()

        self.assertIn(name, [c.name for c in containers])

    def test_create(self):
        """Creates and returns a new container."""
        _, alias = self.create_image()
        config = {
            'name': 'an-container',
            'architecture': '2',
            'profiles': ['default'],
            'ephemeral': True,
            'config': {'limits.cpu': '2'},
            'source': {'type': 'image',
                       'alias': alias},
        }
        self.addCleanup(self.delete_container, config['name'])

        container = self.client.containers.create(config, wait=True)

        self.assertEqual(config['name'], container.name)


class TestContainer(IntegrationTestCase):
    """Tests for Client.Container."""

    def setUp(self):
        super(TestContainer, self).setUp()
        name = self.create_container()
        self.container = self.client.containers.get(name)

    def tearDown(self):
        super(TestContainer, self).tearDown()
        self.delete_container(self.container.name)

    def test_save(self):
        """The container is updated to a new config."""
        self.container.config['limits.cpu'] = '1'
        self.container.save(wait=True)

        self.assertEqual('1', self.container.config['limits.cpu'])
        container = self.client.containers.get(self.container.name)
        self.assertEqual('1', container.config['limits.cpu'])

    def test_rename(self):
        """The container is renamed."""
        name = 'an-renamed-container'
        self.container.rename(name, wait=True)

        self.assertEqual(name, self.container.name)
        container = self.client.containers.get(name)
        self.assertEqual(name, container.name)

    def test_delete(self):
        """The container is deleted."""
        self.container.delete(wait=True)

        self.assertRaises(
            exceptions.LXDAPIException,
            self.client.containers.get, self.container.name)

    def test_start_stop(self):
        """The container is started and then stopped."""
        # NOTE: rockstar (15 Feb 2016) - I don't care for the
        # multiple assertions here, but it's a okay-ish way
        # to test what we need.
        self.container.start(wait=True)

        self.assertEqual('Running', self.container.status)
        container = self.client.containers.get(self.container.name)
        self.assertEqual('Running', container.status)

        self.container.stop(wait=True)

        self.assertEqual('Stopped', self.container.status)
        container = self.client.containers.get(self.container.name)
        self.assertEqual('Stopped', container.status)

    def test_snapshot(self):
        """A container snapshot is made, renamed, and deleted."""
        # NOTE: rockstar (15 Feb 2016) - Once again, multiple things
        # asserted in the same test.
        name = 'an-snapshot'
        snapshot = self.container.snapshots.create(name, wait=True)

        self.assertEqual(
            [name], [s.name for s in self.container.snapshots.all()])

        new_name = 'an-other-snapshot'
        snapshot.rename(new_name, wait=True)

        self.assertEqual(
            [new_name], [s.name for s in self.container.snapshots.all()])

        snapshot.delete(wait=True)

        self.assertEqual([], self.container.snapshots.all())

    def test_put_get_file(self):
        """A file is written to the container and then read."""
        filepath = '/tmp/an_file'
        data = b'abcdef'

        retval = self.container.files.put(filepath, data)

        self.assertTrue(retval)

        contents = self.container.files.get(filepath)

        self.assertEqual(data, contents)

    def test_execute(self):
        """A command is executed on the container."""
        self.container.start(wait=True)
        self.addCleanup(self.container.stop, wait=True)

        result = self.container.execute(['echo', 'test'])

        self.assertEqual(0, result.exit_code)
        self.assertEqual('test\n', result.stdout)
        self.assertEqual('', result.stderr)

    def test_publish(self):
        """A container is published."""
        image = self.container.publish(wait=True)

        self.assertIn(
            image.fingerprint,
            [i.fingerprint for i in self.client.images.all()])



# Copyright (c) 2016 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from pylxd import exceptions


class Operation(object):
    """A LXD operation."""

    __slots__ = [
        '_client',
        'class', 'created_at', 'err', 'id', 'may_cancel', 'metadata',
        'resources', 'status', 'status_code', 'updated_at']

    @classmethod
    def wait_for_operation(cls, client, operation_id):
        """Get an operation and wait for it to complete."""
        operation = cls.get(client, operation_id)
        operation.wait()
        return cls.get(client, operation.id)

    @classmethod
    def get(cls, client, operation_id):
        """Get an operation."""
        if operation_id.startswith('/'):
            operation_id = operation_id.split('/')[-1]
        response = client.api.operations[operation_id].get()
        return cls(_client=client, **response.json()['metadata'])

    def __init__(self, **kwargs):
        super(Operation, self).__init__()
        for key, value in kwargs.items():
            setattr(self, key, value)

    def wait(self):
        """Wait for the operation to complete and return."""
        response = self._client.api.operations[self.id].wait.get()

        try:
            if response.json()['metadata']['status'] == 'Failure':
                raise exceptions.LXDAPIException(response)
        except KeyError:
            # Support for legacy LXD
            pass



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from pylxd.deprecated import base


class LXDCertificate(base.LXDBase):

    def certificate_list(self):
        (state, data) = self.connection.get_object('GET', '/1.0/certificates')
        return [certificate.split('/1.0/certificates/')[-1]
                for certificate in data['metadata']]

    def certificate_show(self, fingerprint):
        return self.connection.get_object('GET', '/1.0/certificates/%s'
                                          % fingerprint)

    def certificate_create(self, certificate):
        return self.connection.get_status('POST', '/1.0/certificates',
                                          json.dumps(certificate))

    def certificate_delete(self, fingerprint):
        return self.connection.get_status('DELETE', '/1.0/certificates/%s'
                                          % fingerprint)



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ddt import ddt
import inspect
import mock
import six
from six.moves import cStringIO
from six.moves import http_client
import socket
import unittest

from pylxd.deprecated import connection
from pylxd.deprecated import exceptions
from pylxd.deprecated.tests import annotated_data

if six.PY3:
    from io import BytesIO


@ddt
class LXDInitConnectionTest(unittest.TestCase):

    @mock.patch('socket.socket')
    @mock.patch.object(http_client.HTTPConnection, '__init__')
    def test_http_connection(self, mc, ms):
        conn = connection.UnixHTTPConnection('/', 'host', 1234)
        if six.PY3:
            mc.assert_called_once_with(
                conn, 'host', port=1234, timeout=None)
        else:
            mc.assert_called_once_with(
                conn, 'host', port=1234, strict=None, timeout=None)
        conn.connect()
        ms.assert_called_once_with(socket.AF_UNIX, socket.SOCK_STREAM)
        ms.return_value.connect.assert_called_once_with('/')

    @mock.patch('os.environ', {'HOME': '/home/foo'})
    @mock.patch('ssl.wrap_socket')
    @mock.patch('socket.create_connection')
    def test_https_connection(self, ms, ml):
        conn = connection.HTTPSConnection('host', 1234)
        with mock.patch.object(conn, '_tunnel') as mc:
            conn.connect()
            self.assertFalse(mc.called)
            ms.assert_called_once_with(
                ('host', 1234), socket._GLOBAL_DEFAULT_TIMEOUT, None)
            ml.assert_called_once_with(
                ms.return_value,
                certfile='/home/foo/.config/lxc/client.crt',
                keyfile='/home/foo/.config/lxc/client.key',
                ssl_version=connection.DEFAULT_TLS_VERSION,
            )

    @mock.patch('os.environ', {'HOME': '/home/foo'})
    @mock.patch('ssl.wrap_socket')
    @mock.patch('socket.create_connection')
    def test_https_proxy_connection(self, ms, ml):
        conn = connection.HTTPSConnection('host', 1234)
        conn._tunnel_host = 'host'
        with mock.patch.object(conn, '_tunnel') as mc:
            conn.connect()
            self.assertTrue(mc.called)
            ms.assert_called_once_with(
                ('host', 1234), socket._GLOBAL_DEFAULT_TIMEOUT, None)
            ml.assert_called_once_with(
                ms.return_value,
                certfile='/home/foo/.config/lxc/client.crt',
                keyfile='/home/foo/.config/lxc/client.key',
                ssl_version=connection.DEFAULT_TLS_VERSION)

    @mock.patch('pylxd.deprecated.connection.HTTPSConnection')
    @mock.patch('pylxd.deprecated.connection.UnixHTTPConnection')
    @annotated_data(
        ('unix', (None,), {}, '/var/lib/lxd/unix.socket'),
        ('unix_path', (None,),
         {'LXD_DIR': '/fake/'}, '/fake/unix.socket'),
        ('https', ('host',), {}, ''),
        ('https_port', ('host', 1234), {}, ''),
    )
    def test_get_connection(self, mode, args, env, path, mc, ms):
        with mock.patch('os.environ', env):
            conn = connection.LXDConnection(*args).get_connection()
            if mode.startswith('unix'):
                self.assertEqual(mc.return_value, conn)
                mc.assert_called_once_with(path)
            elif mode.startswith('https'):
                self.assertEqual(ms.return_value, conn)
                ms.assert_called_once_with(
                    args[0], len(args) == 2 and args[1] or 8443)


class FakeResponse(object):

    def __init__(self, status, data):
        self.status = status
        if six.PY3:
            self.read = BytesIO(six.b(data)).read
        else:
            self.read = cStringIO(data).read


@ddt
@mock.patch('pylxd.deprecated.connection.LXDConnection.get_connection')
class LXDConnectionTest(unittest.TestCase):

    def setUp(self):
        super(LXDConnectionTest, self).setUp()
        self.conn = connection.LXDConnection()

    @annotated_data(
        ('null', (200, '{}'), exceptions.PyLXDException),
        ('200', (200, '{"foo": "bar"}'), (200, {'foo': 'bar'})),
        ('202', (202, '{"status_code": 100}'), (202, {'status_code': 100})),
        ('500', (500, '{"foo": "bar"}'), exceptions.APIError),
    )
    def test_get_object(self, tag, effect, result, mg):
        mg.return_value.getresponse.return_value = FakeResponse(*effect)
        if inspect.isclass(result):
            self.assertRaises(result, self.conn.get_object)
        else:
            self.assertEqual(result, self.conn.get_object())

    @annotated_data(
        ('null', (200, '{}'), exceptions.PyLXDException),
        ('200', (200, '{"foo": "bar"}'), True),
        ('202', (202, '{"status_code": 100}'), True),
        ('200', (200, '{"error": "bar"}'), exceptions.APIError),
        ('500', (500, '{"foo": "bar"}'), False),
    )
    def test_get_status(self, tag, effect, result, mg):
        mg.return_value.getresponse.return_value = FakeResponse(*effect)
        if inspect.isclass(result):
            self.assertRaises(result, self.conn.get_status)
        else:
            self.assertEqual(result, self.conn.get_status())

    @annotated_data(
        ('null', (200, ''), exceptions.PyLXDException),
        ('200', (200, '{"foo": "bar"}'), six.b('{"foo": "bar"}')),
        ('500', (500, '{"foo": "bar"}'),
         exceptions.PyLXDException),
    )
    def test_get_raw(self, tag, effect, result, mg):
        mg.return_value.getresponse.return_value = FakeResponse(*effect)
        if inspect.isclass(result):
            self.assertRaises(result, self.conn.get_raw)
        else:
            self.assertEqual(result, self.conn.get_raw())

    @mock.patch('pylxd.deprecated.connection.WebSocketClient')
    @annotated_data(
        ('fake_host', 'wss://fake_host:8443'),
        (None, 'ws+unix:///var/lib/lxd/unix.socket')
    )
    def test_get_ws(self, host, result, mock_ws, _):
        conn = connection.LXDConnection(host)

        conn.get_ws('/fake/path')

        mock_ws.assert_has_calls([mock.call(result), mock.call().connect()])



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ddt import data
from ddt import ddt
import mock

from pylxd.deprecated import connection

from pylxd.deprecated.tests import annotated_data
from pylxd.deprecated.tests import fake_api
from pylxd.deprecated.tests import LXDAPITestBase


@mock.patch.object(connection.LXDConnection, 'get_object',
                   return_value=(200, fake_api.fake_profile()))
class LXDAPIProfilesTestObject(LXDAPITestBase):

    def test_list_profiles(self, ms):
        ms.return_value = ('200', fake_api.fake_profile_list())
        self.assertEqual(
            ['fake-profile'],
            self.lxd.profile_list())
        ms.assert_called_with('GET',
                              '/1.0/profiles')

    def test_profile_show(self, ms):
        self.assertEqual(
            ms.return_value, self.lxd.profile_show('fake-profile'))
        ms.assert_called_with('GET',
                              '/1.0/profiles/fake-profile')


@ddt
@mock.patch.object(connection.LXDConnection, 'get_status', return_value=True)
class LXDAPIProfilesTestStatus(LXDAPITestBase):

    @data(True, False)
    def test_profile_defined(self, defined, ms):
        ms.return_value = defined
        self.assertEqual(defined, self.lxd.profile_defined('fake-profile'))
        ms.assert_called_with('GET',
                              '/1.0/profiles/fake-profile')

    @annotated_data(
        ('create', 'POST', '', ('fake config',), ('"fake config"',)),
        ('update', 'PUT', '/fake-profile',
         ('fake-profile', 'fake config',), ('"fake config"',)),
        ('rename', 'POST', '/fake-profile',
         ('fake-profile', 'fake config',), ('"fake config"',)),
        ('delete', 'DELETE', '/fake-profile', ('fake-profile',), ()),
    )
    def test_profile_operations(self, method, http, path, args, call_args, ms):
        self.assertTrue(
            getattr(self.lxd, 'profile_' + method)(*args))
        ms.assert_called_with(http,
                              '/1.0/profiles' + path,
                              *call_args)



# Copyright (c) 2016 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import binascii

from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.serialization import Encoding

from pylxd.models import _model as model


class Certificate(model.Model):
    """A LXD certificate."""

    certificate = model.Attribute()
    fingerprint = model.Attribute()
    type = model.Attribute()

    @classmethod
    def get(cls, client, fingerprint):
        """Get a certificate by fingerprint."""
        response = client.api.certificates[fingerprint].get()

        return cls(client, **response.json()['metadata'])

    @classmethod
    def all(cls, client):
        """Get all certificates."""
        response = client.api.certificates.get()

        certs = []
        for cert in response.json()['metadata']:
            fingerprint = cert.split('/')[-1]
            certs.append(cls(client, fingerprint=fingerprint))
        return certs

    @classmethod
    def create(cls, client, password, cert_data):
        """Create a new certificate."""
        cert = x509.load_pem_x509_certificate(cert_data, default_backend())
        base64_cert = cert.public_bytes(Encoding.PEM).decode('utf-8')
        # STRIP OUT CERT META "-----BEGIN CERTIFICATE-----"
        base64_cert = '\n'.join(base64_cert.split('\n')[1:-2])
        data = {
            'type': 'client',
            'certificate': base64_cert,
            'password': password,
        }
        client.api.certificates.post(json=data)

        # XXX: rockstar (08 Jun 2016) - Please see the open lxd bug here:
        # https://github.com/lxc/lxd/issues/2092
        fingerprint = binascii.hexlify(
            cert.fingerprint(hashes.SHA256())).decode('utf-8')
        return cls.get(client, fingerprint)

    @property
    def api(self):
        return self.client.api.certificates[self.fingerprint]



# Copyright (c) 2016 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from pylxd import exceptions, models
from pylxd.tests import testing


class TestOperation(testing.PyLXDTestCase):
    """Tests for pylxd.models.Operation."""

    def test_get(self):
        """Return an operation."""
        name = 'operation-abc'

        an_operation = models.Operation.get(self.client, name)

        self.assertEqual(name, an_operation.id)

    def test_get_full_path(self):
        """Return an operation even if the full path is specified."""
        name = '/1.0/operations/operation-abc'

        an_operation = models.Operation.get(self.client, name)

        self.assertEqual('operation-abc', an_operation.id)

    def test_wait_with_error(self):
        """If the operation errors, wait raises an exception."""
        def error(request, context):
            context.status_code = 200
            return {
                'type': 'sync',
                'metadata': {
                    'status': 'Failure',
                    'err': 'Keep your foot off the blasted samoflange.',
                }}
        self.add_rule({
            'json': error,
            'method': 'GET',
            'url': r'^http://pylxd.test/1.0/operations/operation-abc/wait$',  # NOQA
        })

        name = '/1.0/operations/operation-abc'

        an_operation = models.Operation.get(self.client, name)

        self.assertRaises(exceptions.LXDAPIException, an_operation.wait)



import unittest

import mock_services

from pylxd.client import Client
from pylxd.tests import mock_lxd


def requires_ws4py(f):
    """Marks a test as requiring ws4py.

    As ws4py is an optional dependency, some tests should
    be skipped, as they won't run properly if ws4py is not
    installed.
    """
    try:
        import ws4py  # NOQA
        return f
    except ImportError:
        return unittest.skip('ws4py is not installed')(f)


class PyLXDTestCase(unittest.TestCase):
    """A test case for handling mocking of LXD services."""

    def setUp(self):
        mock_services.update_http_rules(mock_lxd.RULES)
        mock_services.start_http_mock()

        self.client = Client(endpoint='http://pylxd.test')

    def tearDown(self):
        mock_services.stop_http_mock()

    def add_rule(self, rule):
        """Add a rule to the mock LXD service."""
        mock_services.update_http_rules([rule])



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ddt import data
from ddt import unpack
import unittest

from pylxd.deprecated import api


class LXDAPITestBase(unittest.TestCase):

    def setUp(self):
        super(LXDAPITestBase, self).setUp()
        self.lxd = api.API()


def annotated_data(*args):
    class List(list):
        pass

    new_args = []

    for arg in args:
        new_arg = List(arg)
        new_arg.__name__ = arg[0]
        new_args.append(new_arg)

    return lambda func: data(*new_args)(unpack(func))



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import print_function

from pylxd.deprecated import base
from pylxd.deprecated import exceptions


class LXDHost(base.LXDBase):

    def host_ping(self):
        try:
            return self.connection.get_status('GET', '/1.0')
        except Exception as e:
            msg = 'LXD service is unavailable. %s' % e
            raise exceptions.PyLXDException(msg)

    def host_info(self):
        (state, data) = self.connection.get_object('GET', '/1.0')

        return {
            'lxd_api_compat_level':
                self.get_lxd_api_compat(data.get('metadata')),
            'lxd_trusted_host':
                self.get_lxd_host_trust(data.get('metadata')),
            'lxd_backing_fs':
                self.get_lxd_backing_fs(data.get('metadata')),
            'lxd_driver':
                self.get_lxd_driver(data.get('metadata')),
            'lxd_version':
                self.get_lxd_version(data.get('metadata')),
            'lxc_version':
                self.get_lxc_version(data.get('metadata')),
            'kernel_version':
                self.get_kernel_version(data.get('metadata'))
        }

    def get_lxd_api_compat(self, data):
        try:
            if data is None:
                (state, data) = self.connection.get_object('GET', '/1.0')
                data = data.get('metadata')
            return data['api_compat']
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))

    def get_lxd_host_trust(self, data):
        try:
            if data is None:
                (state, data) = self.connection.get_object('GET', '/1.0')
                data = data.get('metadata')
            return True if data['auth'] == 'trusted' else False
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))

    def get_lxd_backing_fs(self, data):
        try:
            if data is None:
                (state, data) = self.connection.get_object('GET', '/1.0')
                data = data.get('metadata')
            return data['environment']['backing_fs']
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))

    def get_lxd_driver(self, data):
        try:
            if data is None:
                (state, data) = self.connection.get_object('GET', '/1.0')
                data = data.get('metadata')
            return data['environment']['driver']
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))

    def get_lxc_version(self, data):
        try:
            if data is None:
                (state, data) = self.connection.get_object('GET', '/1.0')
                data = data.get('metadata')
            return data['environment']['lxc_version']
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))

    def get_lxd_version(self, data):
        try:
            if data is None:
                (state, data) = self.connection.get_object('GET', '/1.0')
                data = data.get('metadata')
            return float(data['environment']['lxd_version'])
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))

    def get_kernel_version(self, data):
        try:
            if data is None:
                (state, data) = self.connection.get_object('GET', '/1.0')
                data = data.get('metadata')
            return data['environment']['kernel_version']
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))

    def get_certificate(self):
        try:
            (state, data) = self.connection.get_object('GET', '/1.0')
            data = data.get('metadata')
            return data['environment']['certificate']
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))

    def host_config(self):
        try:
            (state, data) = self.connection.get_object('GET', '/1.0')
            return data.get('metadata')
        except exceptions.PyLXDException as e:
            print('Handling run-time error: {}'.format(e))



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
from ddt import ddt
import mock

from pylxd.deprecated import connection

from pylxd.deprecated.tests import annotated_data
from pylxd.deprecated.tests import fake_api
from pylxd.deprecated.tests import LXDAPITestBase


@ddt
@mock.patch.object(connection.LXDConnection, 'get_object',
                   return_value=(200, fake_api.fake_operation()))
class LXDAPIOperationTestObject(LXDAPITestBase):

    def test_list_operations(self, ms):
        ms.return_value = ('200', fake_api.fake_operation_list())
        self.assertEqual(
            ['/1.0/operations/1234'],
            self.lxd.list_operations())
        ms.assert_called_with('GET',
                              '/1.0/operations')

    def test_operation_info(self, ms):
        ms.return_value = ('200', fake_api.fake_operation())
        self.assertEqual(
            ms.return_value, self.lxd.operation_info('/1.0/operations/1234'))
        ms.assert_called_with('GET',
                              '/1.0/operations/1234')

    @annotated_data(
        ('create_time',
         datetime.datetime.utcfromtimestamp(1433876844)
         .strftime('%Y-%m-%d %H:%M:%S')),
        ('update_time',
         datetime.datetime.utcfromtimestamp(1433876843)
         .strftime('%Y-%m-%d %H:%M:%S')),
        ('status', 'Running'),
    )
    def test_operation_show(self, method, expected, ms):
        call = getattr(self.lxd, 'operation_show_' + method)
        self.assertEqual(expected, call('/1.0/operations/1234'))
        ms.assert_called_with('GET',
                              '/1.0/operations/1234')


@ddt
@mock.patch.object(connection.LXDConnection, 'get_status', return_value=True)
class LXDAPIOperationTestStatus(LXDAPITestBase):

    @annotated_data(
        ('operation_delete', 'DELETE', '', ()),
        ('wait_container_operation', 'GET',
         '/wait?status_code=200&timeout=30', ('200', '30')),
    )
    def test_operation_actions(self, method, http, path, args, ms):
        self.assertTrue(
            getattr(self.lxd, method)('/1.0/operations/1234', *args))
        ms.assert_called_with(http,
                              '/1.0/operations/1234' + path)



# Copyright (c) 2016 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from pylxd import models
from pylxd.tests import testing


class TestNetwork(testing.PyLXDTestCase):
    """Tests for pylxd.models.Network."""

    def test_all(self):
        """A list of all networks are returned."""
        networks = models.Network.all(self.client)

        self.assertEqual(1, len(networks))

    def test_get(self):
        """Return a container."""
        name = 'lo'

        an_network = models.Network.get(self.client, name)

        self.assertEqual(name, an_network.name)

    def test_partial(self):
        """A partial network is synced."""
        an_network = models.Network(self.client, name='lo')

        self.assertEqual('loopback', an_network.type)

    def test_delete(self):
        """delete is not implemented in networks."""
        an_network = models.Network(self.client, name='lo')

        with self.assertRaises(NotImplementedError):
            an_network.delete()

    def test_save(self):
        """save is not implemented in networks."""
        an_network = models.Network(self.client, name='lo')

        with self.assertRaises(NotImplementedError):
            an_network.save()



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from pylxd import api
from pylxd import exceptions as lxd_exceptions


def upload_image(image):
    alias = '{}/{}/{}/{}'.format(image['os'],
                                 image['release'],
                                 image['arch'],
                                 image['variant'])
    lxd = api.API()
    imgs = api.API(host='images.linuxcontainers.org')
    d = imgs.alias_show(alias)

    meta = d[1]['metadata']
    tgt = meta['target']

    try:
        lxd.alias_update(meta)
    except lxd_exceptions.APIError as ex:
        if ex.status_code == 404:
            lxd.alias_create(meta)

    return tgt


def delete_image(image):
    lxd = api.API()
    lxd.image_delete(image)



# Copyright (c) 2015 Canonical Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ddt import ddt
import json
import mock

from pylxd.deprecated import connection

from pylxd.deprecated.tests import annotated_data
from pylxd.deprecated.tests import fake_api
from pylxd.deprecated.tests import LXDAPITestBase


@ddt
class LXDAPICertificateTest(LXDAPITestBase):

    def test_list_certificates(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_certificate_list())
            self.assertEqual(
                ['ABCDEF01'],
                self.lxd.certificate_list())
            ms.assert_called_with('GET',
                                  '/1.0/certificates')

    def test_certificate_show(self):
        with mock.patch.object(connection.LXDConnection, 'get_object') as ms:
            ms.return_value = ('200', fake_api.fake_certificate())
            self.assertEqual(
                ms.return_value, self.lxd.certificate_show('ABCDEF01'))
            ms.assert_called_with('GET',
                                  '/1.0/certificates/ABCDEF01')

    @annotated_data(
        ('delete', 'DELETE', '/ABCDEF01'),
        ('create', 'POST', '', (json.dumps('ABCDEF01'),)),
    )
    def test_certificate_operations(self, method, http, path, call_args=()):
        with mock.patch.object(connection.LXDConnection, 'get_status') as ms:
            ms.return_value = True
            self.assertTrue(
                getattr(self.lxd, 'certificate_' + method)('ABCDEF01'))
            ms.assert_called_with(http,
                                  '/1.0/certificates' + path,
                                  *call_args)
